/* $Id: parseargs.cpp,v 1.3 2007-09-03 07:20:18 aehyvari Exp $ */
/* start[copyright.txt] */
/* Including data from file copyright.txt */
/* file opened */
/* Reading 775 bytes of data */
/* Start of include */
/*
 * Copyright 2007 Antti Hyvrinen
 *
 * This file is part of GridJM.
 *
 * GridJM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GridJM is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GridJM; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/* End of include */
/* stop[copyright.txt] */

#include <stdio.h>
#include <string>
#include <iostream>
#include <getopt.h>
#include "parseargs.h"
#include "version.h"

using namespace std;

char *optargs = "ad:e:hi:j:m:n:p:q:r:t:u:vx:";

int optind, opterr, optopt;

char * usage_string = "Gridjm usage:\n\
  --automatic                Try to automatically guess the available\n\
                              resources in the grid (experimental)\n\
  --cluster-update int,      Cluster update interval (seconds)\n\
    -c int                    (3600) \n\
  --dir str, -d str          Use the download directory given as argument\n\
                              (default is /tmp/ngdownload/)\n\
  --errdir dir, -e dir       Receive failed jobs to directory (default no)\n\
  --help, -h                 This help text\n\
  --instrumentation str,     Instrumentation file (default /tmp/gridjm.ins)\n\
    -i str                   \n\
  --jobstall int, -j int     Job state stall timeout (seconds)\n\
                              (600)\n\
  --maxjobs int, -m int      Maximum number of simultaneous jobs requested\n\
  --ngget str, -n str        Use ngget program given as argument\n\
                              (default is ngget)\n\
  --port int, -p int         Use tcp port number given as argument (12345)\n\
  --queue-update int, -q int Queue update period (seconds) (300)\n\
  --pidfile str, -r str      Pid file name (default /tmp/gridjm.pid)\n\
  --interval int, -t int     Update interval for jobs in grid (seconds)\n\
  --useuds str, -u str       Use uds for communication\n\
                              (default is not to use)\n\
  --retry int, -x int        Number of submission retries (1)\n\
  --version, -v              Show the version number\n\
";

void parseargs(int argc, char **argv, struct arguments *args) {
    int c;
    int digit_optind = 0;

    // Default values
    args->error = false;
    args->geterrors = false;
    args->errordir = "";
    args->useuds = false;
    args->socketstring = "";
    args->port = 12345;
    args->maxjobs = 64;
    args->ngget = "ngget";
    args->dir = "/tmp/ngdownload";
    args->pidfile = "/tmp/gridjm.pid";
    args->instrumentation = "/tmp/gridjm.ins";
    args->interval = 30;
    args->clupdp = 3600;
    args->qupdp = 300;
    args->s_timeout = 600;
    args->retry = 1;
    args->automatic = false;

    while (1) {
        int this_option_optind = (optind ? optind : 1);
        int option_index = 0;
        static struct option long_options[] = {
            {"automatic", 0, 0, 'a'},
            {"cluster-update", 1, 0, 'c'},
            {"dir", 1, 0, 'd'},
            {"errdir", 1, 0, 'e'},
            {"help", 0, 0, 'h'},
            {"instrumentation", 1, 0, 'i'},
            {"jobstall", 1, 0, 'j'},
            {"maxjobs", 1, 0, 'm'},
            {"ngget", 1, 0, 'n'},
            {"port", 1, 0, 'p'},
            {"queue-update", 1, 0, 'q'},
            {"pidfile", 1, 0, 'r'},
            {"interval", 1, 0, 't'},
            {"useuds", 1, 0, 'u'},
            {"retry", 1, 0, 'x'},
            {"version", 0, 0, 'v'},
            {0, 0, 0, 0}
        };

        c = getopt_long(argc, argv, optargs,
                long_options, &option_index);
        if (c == -1)
            break;

        switch (c) {
            case 0:
                cout << "Option " << long_options[option_index].name;
                if (optarg)
                    cout << " with arg " << optarg;
                cout << endl;
                args->error = true;
                break;

            case 'a':
                cout << "Warning: Automatic discovery is experimental" << endl;
                args->automatic = true;
                break;

            case 'c':
                args->clupdp = atoi(optarg);
                break;

            case 'd':
                args->dir = optarg;
                break;

            case 'e':
                args->geterrors = true;
                args->errordir = optarg;
                args->errclfile = (char*) malloc(strlen(args->errordir) + 9);
                strncpy(args->errclfile,
                        args->errordir, strlen(args->errordir));
                strncpy(args->errclfile + strlen(args->errordir),
                        "/jobs.txt", strlen(args->errordir) + 9);
                break;

            case 'h':
                cout << version_string << endl;
                cout << usage_string;
                args->error = true;
                break;

            case 'i':
                args->instrumentation = optarg;
                break;

            case 'j':
                args->s_timeout = atoi(optarg);
                break;

            case 'm':
                args->maxjobs = atoi(optarg);
                break;

            case 'n':
                args->ngget = optarg;
                break;

            case 'p':
                cout << "Using port number " << optarg << endl;
                args->port = atoi(optarg);
                break;

            case 'q':
                args->qupdp = atoi(optarg);
                break;

            case 'r':
                args->pidfile = optarg;
                break;

            case 't':
                args->interval = atoi(optarg);
                break;

            case 'u':
                cout << "Using socket name " << optarg << endl;
                args->useuds = true;
                args->socketstring = optarg;
                break;

            case 'v':
                cout << version_string << endl;
                args->error = true;
                break;

            case 'x':
                cout << "Using retry " << optarg << endl;
                args->retry = atoi(optarg);
                break;

            case '?':
                cout << "Unknown option " << argv[1] << endl;
                args->error = true;
                break;

            case ':':
                cout << "Option requires an argument " << \
                    argv[1] << endl;
                args->error = true;
                break;

            default:
                cout << "Getopt returned strange value " << endl;
                args->error = true;
        }
    }
    return;
}
